@@ -76,14 +76,12 @@ fn array_layout<D: Dimension>(dim: &D, strides: &D) -> Layout
     }
 }
 
-impl<S, D> ArrayBase<S, D>
-where
-    S: RawData,
-    D: Dimension,
+impl<A, D> LayoutRef<A, D>
+where D: Dimension
 {
     pub(crate) fn layout_impl(&self) -> Layout
     {
-        array_layout(&self.dim, &self.strides)
+        array_layout(self._dim(), self._strides())
     }
 }
 
@@ -96,8 +94,8 @@ where
     fn broadcast_unwrap(self, shape: E) -> Self::Output
     {
         #[allow(clippy::needless_borrow)]
-        let res: ArrayView<'_, A, E::Dim> = (&self).broadcast_unwrap(shape.into_dimension());
-        unsafe { ArrayView::new(res.ptr, res.dim, res.strides) }
+        let res: ArrayView<'_, A, E::Dim> = (*self).broadcast_unwrap(shape.into_dimension());
+        unsafe { ArrayView::new(res.parts.ptr, res.parts.dim, res.parts.strides) }
     }
     private_impl! {}
 }
@@ -762,7 +760,8 @@ macro_rules! map_impl {
 
             pub(crate) fn map_collect_owned<S, R>(self, f: impl FnMut($($p::Item,)* ) -> R)
                 -> ArrayBase<S, D>
-                where S: DataOwned<Elem = R>
+                where
+                    S: DataOwned<Elem = R>,
             {
                 // safe because: all elements are written before the array is completed
 
